from typing import List
from .auth import Auth


class Bridge:
    """Class that represents a Blum Bridge object in the API."""

    def __init__(self, raw_data: dict, auth: Auth):
        self.raw_data = raw_data
        self.auth = auth


    @property
    def bridgeId(self) -> str:
        """Return the id of the bridge."""
        return self.raw_data["bridgeId"]

    @property
    def bridgeAppVersion(self) -> str:
        """Return the bridgeAppVersion of the bridge."""
        return self.raw_data["bridgeAppVersion"]
    
    @property
    def dps(self) -> str:
        """Return the dps of the bridge."""
        return self.raw_data["dps"]

    @property
    def iothub(self) -> str:
        """Return the iothub of the bridge."""
        return self.raw_data["iothub"]
    
    @property
    def rfVersion(self) -> str:
        """Return the rfVersion of the bridge."""
        return self.raw_data["rfVersion"]


    def __str__(self):
        return f'Bridge(bridgeId={self.bridgeId}, bridgeAppVersion={self.bridgeAppVersion}, dps={self.dps}, iothub={self.iothub}, rfVersion={self.rfVersion})'

    def __repr__(self):
        return self.__str__()


from typing import List
from .auth import Auth
from .module import Module
from .bridge import Bridge


class BridgeAPI:

    def __init__(self, auth: Auth):        
        """Initialize the API and store the auth so we can make requests."""        
        self.auth = auth

    async def async_get_modules(self) -> List[Module]:
        response = await self.auth.request("get", "modules/show")
        response.raise_for_status()
        return [Module(module_data, self.auth) for module_data in await response.json()]

    async def async_get_module(self, moduleId: str) -> List[Module]:
        response = await self.auth.request("get", f"modules/show?id={moduleId}")
        response.raise_for_status()
        #ToDo: Should return not a list
        #return Module(await response.json(), self.auth)
        return[Module(module_data, self.auth) for module_data in await response.json()]

    async def async_get_bridge(self) -> Bridge:
        response = await self.auth.request("get", "check")
        response.raise_for_status()
        return Bridge(await response.json(), self.auth)


from typing import List
from .auth import Auth


class Module:
    """Class that represents a Module object in the API."""

    def __init__(self, raw_data: dict, auth: Auth):
        self.raw_data = raw_data
        self.auth = auth

    @property
    def id(self) -> str:
        """Return the id of the module."""
        return self.raw_data["id"]

    @property
    def name(self) -> str:
        """Return the name of the module."""
        return self.raw_data["name"]

    @property
    def aliases(self) -> dict:
        """Return the aliases of the module."""
        return self.raw_data["aliases"]

    @property
    def canClose(self) -> bool:
        """Return if the module is closeable."""
        return self.raw_data["canClose"]

    @property
    def source(self) -> str:
        """Return the source of the module."""
        return self.raw_data["source"]

    @property
    def state(self) -> str:
        """Return the state of the module."""
        return self.raw_data["state"]

    @property
    def targetName(self) -> str:
        """Return the targetName of the module."""
        return self.raw_data["targetName"]

    @property
    def type(self) -> str:
        """Return the type of the module."""
        return self.raw_data["type"]

    async def async_control(self, command: str):
        """Control the module."""

        response = await self.auth.request(
            "post", f"modules/command", json={"id": self.id, "command": command}
        )
        response.raise_for_status()
        #This should be made possible. Seccond update-Call necessary
        #self.raw_data = await response.json()
        #Until this works, little hack to set the state directly correct:
        if(command == "open"):
            self.raw_data["state"] = "open"
        else:
            self.raw_data["state"] = "closed"

    async def async_update(self):
        """Update the module data."""

        response = await self.auth.request("get", f"modules/show?id={self.id}")
        response.raise_for_status()
        moduleList = await response.json()
        self.raw_data = moduleList[0]


    def __str__(self):
        return f'Module(id={self.id}, name={self.name}, state={self.state}, type={self.type})'

    def __repr__(self):
        return self.__str__()



from aiohttp import ClientSession, ClientResponse, BasicAuth
from urllib.parse import urlparse

class Auth:
    """Class to make authenticated requests."""

    def __init__(self, clientSession: ClientSession, host: str, user: str, password: str):        
        """Initialize the auth."""
        
        self.clientSession = clientSession
        self.auth = BasicAuth(login=user, password=password, encoding='utf-8')
        self.host = urlparse(host)
        #If scheme is missing urlparse does not work
        if self.host.scheme is "":
            raise Exception(f"The host: {host} is invalid. Should be look like <scheme>://<hostname> or <scheme>://<hostname>:<port>")

        self.port = self.host.port if self.host.port else 3490
        self.scheme = self.host.scheme if self.host.scheme else "http"
        self.api_prefix = "api/v1"


    async def request(self, method: str, path: str, **kwargs) -> ClientResponse:
        """Make a request."""

        headers = kwargs.get("headers")
        if headers is None:
            headers = {}
        else:
            headers = dict(headers)
        
        return await self.clientSession.request(
            method, f"{self.scheme}://{self.host.hostname}:{self.port}/{self.api_prefix}/{path}", **kwargs, headers=headers, auth=self.auth,
        )